# osh
# Copyright (C) 2005 Jack Orenstein <jao@geophile.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

"""Usage: gen [COUNT [START]]

Generate a sequence of COUNT integers, starting at START. If START is not
specified, then the sequence starts at 0. If no arguments are specified,
then the sequence does not terminate.
"""

import osh.oshobjects

def gen(*args):
    return _Gen().process_args(*args)

class _Gen(osh.oshobjects.Generator):

    _count = None
    _start = None


    # object interface

    def __init__(self):
        osh.oshobjects.Generator.__init__(self, '', (0, 2))

    # BaseOp interface
    
    def doc(self):
        return __doc__

    def setup(self):
        args = self.args()
        self._count = args.next_int()
        self._start = args.next_int()
        if self._start is None:
            self._start = 0
        if self._count is None or args.has_next():
            self.usage()


    # Generator interface
    
    def execute(self):
        if self._count == 0:
            n = 0
            while True:
                self.send(n)
                n += 1
        else:
            for x in xrange(self._start, self._start + self._count):
                self.send(x)
            self.send_complete()
