# osh
# Copyright (C) 2005 Jack Orenstein <jao@geophile.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import tpg
import oshloader
import oshobjects

oshobjects.osh_usage = oshobjects.USAGE_CLI

class oshparser(tpg.VerboseParser):
    r"""
        separator space '#+';

        token at         '@';
        token colon      ':';
        token comma      ',';
        token empty_list '\[\]';
        token begin      '\[';
        token end        '\]';
        token pipe       '\^';
        token dollar     '\$';
        token string     '[^#]+';

        START/c ->
            COMMAND/c
            (dollar                     $ op = load_op('out')
                                        $ op.add_arg('-t')
                                        $ op.args_done()
                                        $ os = oshobjects.Opset(op)
                                        $ c.pipeline().append_opset(os)
            )?
            ;


        COMMAND/c ->
            pipe
                                        $ generator = load_op('stdin')
                                        $ os = oshobjects.Opset(generator)
            PIPELINE/p                  $ c = oshobjects.command(p.prepend_opset(os))
            |
            PIPELINE/p                  $ c = oshobjects.command(p)
            ;

        PIPELINE/p ->
            OPSET/os                    $ p = oshobjects.Pipeline(os)
            (   pipe
                OPSET/os                $ p = p.append_opset(os)
            )*
            ;

        OPSET/os ->
            OP/op                       $ os = oshobjects.Opset(op)
            (   comma
                OP/op                   $ os = os.add_op(op)
            )*
            ;

        OP/op ->
                                        $ label = 'o'
            (LABEL/label
            )?
            (
                OP_NAME/n               $ op = load_op(n)
                (    ARG/a              $ op = op.add_arg(a)
                )*                      $ op = op.args_done()
                |
                begin
                PIPELINE/p              $ op = p
                end
                |
                REMOTE/r                $ op = load_op('remote')
                                        $ op = op.add_arg('-c')
                                        $ op = op.add_arg(r)
                (    ARG/a              $ op = op.add_arg(a)
                )*                      $ op = op.args_done()
                begin
                PIPELINE/p              $ op = op.set_pipeline(p)
                end
                |
                colon
                string/newLabel         $ op = load_op('relabel')
                                        $ op.add_arg(newLabel)
                                        $ op.args_done()
            )
                                        $ op.set_label(label)
            ;

        LABEL/l ->
            string/l                    
            colon
            ;

        OP_NAME/n ->
            string/n
            ;

        REMOTE/r -> 
           at                           $ r = None
            (   string/n                $ r = n
            )?
            ;

        ARG/a ->
            empty_list                  $ a = '[]'
            |
            string/a
            ;
    """

    verbose = 0

def trace(message, object):
    print '%s(%s)' % (message, object)
    return object

def op(tokens):
    labeled = tokens[0].endswith(':')
    if labeled:
        label = tokens[0]
        opName = tokens[1]
        args = tokens[2:]
    else:
        label = 'o'
        opName = tokens[0]
        args = tokens[1:]
    op = load_op(opName)
    op.setLabel(label)
    for arg in args:
        op.add_arg(arg)
    return op

def load_op(opName):
    return oshloader.load_and_create(opName)

def parse(command):
    return oshparser()(command)
